@@ -4,7 +4,6 @@ ggplot2::autoplot
 #' @export
 autoplot.cocoForecast <- function(object, ...){
   
-  
   pl <- ggplot2::ggplot(mapping = ggplot2::aes(x = object$x, y = object$densities_plot)) +
       ggplot2::geom_bar(stat="identity", position="dodge", width=0.04) + 
       ggplot2::labs(title = "Probability mass forecast", x = "Support", y = "Probability mass") +
@@ -19,11 +18,14 @@ plot.cocoForecast <- function(x, ...) {
     x,
     ...
   )
-  print(p)
+  suppressWarnings({print(p)})
 }
 
-#'@export
-print.cocoForecast <- function(x, ...) {
-  print(autoplot(x, ...,))
-  invisible(x)
+#' @export
+plot.cocoForecast <- function(x, ...) {
+  p <- autoplot(
+    x,
+    ...
+  )
+  suppressWarnings({print(p)})
 }
\ No newline at end of file
